@@ -12,7 +12,8 @@
 
 /* standard level flags part 1 (ECX)*/
 #define FLAG_SSE3      0x00000001
-
+#define FLAG_SSSE3     0x00000200
+#define FLAG_AVX       0x1C000000
 /* standard level flags part 2 (EDX) */
 #define FLAG2_MMX       0x00800000
 #define FLAG2_SSE       0x02000000
@@ -22,6 +23,9 @@
 #define XFLAG_MMX      0x00800000
 #define XFLAG_3DNOW    0x80000000
 #define XFLAG_3DNOWEXT 0x40000000
+/* eXtended Control Register 0 */
+#define XCR0FLAG_AVX   0x00000006
+
 
 struct cpuflags
 {
@@ -29,6 +33,7 @@ struct cpuflags
 	unsigned int std;
 	unsigned int std2;
 	unsigned int ext;
+	unsigned int xcr0_lo;
 };
 
 unsigned int getcpuflags(struct cpuflags* cf);
@@ -43,5 +48,8 @@ unsigned int getcpuflags(struct cpuflags* cf);
 #define cpu_sse(s) (FLAG2_SSE & s.std2)
 #define cpu_sse2(s) (FLAG2_SSE2 & s.std2)
 #define cpu_sse3(s) (FLAG_SSE3 & s.std)
+#define cpu_avx(s) ((FLAG_AVX & s.std) == FLAG_AVX && (XCR0FLAG_AVX & s.xcr0_lo) == XCR0FLAG_AVX)
+#define cpu_fast_sse(s) ((((s.id & 0xf00)>>8) == 6 && FLAG_SSSE3 & s.std) /* for Intel/VIA; family 6 CPUs with SSSE3 */ || \
+						   (((s.id & 0xf00)>>8) == 0xf && (((s.id & 0x0ff00000)>>20) > 0 && ((s.id & 0x0ff00000)>>20) != 5))) /* for AMD; family > 0xF CPUs except Bobcat */
 
 #endif
