@@ -8,48 +8,11 @@
 	Currently, this file contains the struct and function to choose an optimization variant and works only when OPT_MULTI is in effect.
 */
 
+#define I_AM_OPTIMIZE
 #include "mpg123lib_intern.h" /* includes optimize.h */
 #include "debug.h"
 
-/* Must match the enum dectype! */
-
-/*
-	It SUCKS having to define these names that way, but compile-time intialization of string arrays is a bitch.
-	GCC doesn't see constant stuff when it's wiggling in front of it!
-	Anyhow: Have a script for that:
-names="generic generic_dither i386 i486 i586 i586_dither MMX 3DNow 3DNowExt AltiVec SSE x86-64"
-for i in $names; do echo "##define dn_${i/-/_} \"$i\""; done
-echo -n "static const char* decname[] =
-{
-	\"auto\"
-	"
-for i in $names; do echo -n ", dn_${i/-/_}"; done
-echo "
-	, \"nodec\"
-};"
-*/
-#define dn_generic "generic"
-#define dn_generic_dither "generic_dither"
-#define dn_i386 "i386"
-#define dn_i486 "i486"
-#define dn_i586 "i586"
-#define dn_i586_dither "i586_dither"
-#define dn_MMX "MMX"
-#define dn_3DNow "3DNow"
-#define dn_3DNowExt "3DNowExt"
-#define dn_AltiVec "AltiVec"
-#define dn_SSE "SSE"
-#define dn_x86_64 "x86-64"
-#define dn_ARM "ARM"
-#define dn_NEON "NEON"
-static const char* decname[] =
-{
-	"auto"
-	, dn_generic, dn_generic_dither, dn_i386, dn_i486, dn_i586, dn_i586_dither, dn_MMX, dn_3DNow, dn_3DNowExt, dn_AltiVec, dn_SSE, dn_x86_64, dn_ARM, dn_NEON
-	, "nodec"
-};
-
-#if (defined OPT_X86) && (defined OPT_MULTI)
+#if ((defined OPT_X86) || (defined OPT_X86_64)) && (defined OPT_MULTI)
 #include "getcpuflags.h"
 static struct cpuflags cpu_flags;
 #else
@@ -64,6 +27,7 @@ static struct cpuflags cpu_flags;
 #define cpu_sse(s)      1
 #define cpu_sse2(s)     1
 #define cpu_sse3(s)     1
+#define cpu_avx(s)      1
 #endif
 
 /* Ugly macros to build conditional synth function array values. */
@@ -166,10 +130,19 @@ enum optdec defdec(void){ return defopt; }
 
 enum optcla decclass(const enum optdec type)
 {
-	return (type == mmx || type == sse || type == dreidnowext || type == x86_64  || type == neon) ? mmxsse : normal;
+	return
+	(
+		   type == mmx
+		|| type == sse
+		|| type == sse_vintage
+		|| type == dreidnowext
+		|| type == dreidnowext_vintage
+		|| type == x86_64
+		|| type == neon
+		|| type == avx
+	) ? mmxsse : normal;
 }
 
-
 static int find_synth(func_synth synth,  const func_synth synths[r_limit][f_limit])
 {
 	enum synth_resample ri;
@@ -182,6 +155,23 @@ static int find_synth(func_synth synth,  const func_synth synths[r_limit][f_limi
 	return FALSE;
 }
 
+
+/* After knowing that it is either vintage or current SSE,
+   this separates the two. In case of non-OPT_MULTI, only one
+   of OPT_SSE and OPT_SSE_VINTAGE is active. */
+static enum optdec sse_or_vintage(mpg123_handle *fr)
+{
+	enum optdec type;
+	type = sse_vintage;
+#	ifdef OPT_SSE
+#	ifdef OPT_MULTI
+	if(fr->cpu_opts.the_dct36 == dct36_sse)
+#	endif
+	type = sse;
+#	endif
+	return type;
+}
+
 /* Determine what kind of decoder is actually active
    This depends on runtime choices which may cause fallback to i386 or generic code. */
 static int find_dectype(mpg123_handle *fr)
@@ -199,14 +189,35 @@ static int find_dectype(mpg123_handle *fr)
 
 	if(FALSE) ; /* Just to initialize the else if ladder. */
 #ifndef NO_16BIT
-#ifdef OPT_3DNOWEXT
-	else if(basic_synth == synth_1to1_3dnowext) type = dreidnowext;
+#if defined(OPT_3DNOWEXT) || defined(OPT_3DNOWEXT_VINTAGE)
+	else if(basic_synth == synth_1to1_3dnowext)
+	{
+		type = dreidnowext;
+#		ifdef OPT_3DNOWEXT_VINTAGE
+#		ifdef OPT_MULTI
+		if(fr->cpu_opts.the_dct36 == dct36_3dnowext)
+#		endif
+		type = dreidnowext_vintage;
+#		endif
+	}
 #endif
-#ifdef OPT_SSE
-	else if(basic_synth == synth_1to1_sse) type = sse;
+#if defined(OPT_SSE) || defined(OPT_SSE_VINTAGE)
+	else if(basic_synth == synth_1to1_sse)
+	{
+		type = sse_or_vintage(fr);
+	}
 #endif
-#ifdef OPT_3DNOW
-	else if(basic_synth == synth_1to1_3dnow) type = dreidnow;
+#if defined(OPT_3DNOW) || defined(OPT_3DNOW_VINTAGE)
+	else if(basic_synth == synth_1to1_3dnow)
+	{
+		type = dreidnow;
+#		ifdef OPT_3DNOW_VINTAGE
+#		ifdef OPT_MULTI
+		if(fr->cpu_opts.the_dct36 == dct36_3dnow)
+#		endif
+		type = dreidnow_vintage;
+#		endif
+	}
 #endif
 #ifdef OPT_MMX
 	else if(basic_synth == synth_1to1_mmx) type = mmx;
@@ -223,6 +234,9 @@ static int find_dectype(mpg123_handle *fr)
 #ifdef OPT_X86_64
 	else if(basic_synth == synth_1to1_x86_64) type = x86_64;
 #endif
+#ifdef OPT_AVX
+	else if(basic_synth == synth_1to1_avx) type = avx;
+#endif
 #ifdef OPT_ARM
 	else if(basic_synth == synth_1to1_arm) type = arm;
 #endif
@@ -244,12 +258,18 @@ static int find_dectype(mpg123_handle *fr)
 #endif /* 16bit */
 
 #ifndef NO_REAL
-#ifdef OPT_SSE
-	else if(basic_synth == synth_1to1_real_sse) type = sse;
+#if defined(OPT_SSE) || defined(OPT_SSE_VINTAGE)
+	else if(basic_synth == synth_1to1_real_sse)
+	{
+		type = sse_or_vintage(fr);
+	}
 #endif
 #ifdef OPT_X86_64
 	else if(basic_synth == synth_1to1_real_x86_64) type = x86_64;
 #endif
+#ifdef OPT_AVX
+	else if(basic_synth == synth_1to1_real_avx) type = avx;
+#endif
 #ifdef OPT_ALTIVEC
 	else if(basic_synth == synth_1to1_real_altivec) type = altivec;
 #endif
@@ -260,12 +280,18 @@ static int find_dectype(mpg123_handle *fr)
 #endif /* real */
 
 #ifndef NO_32BIT
-#ifdef OPT_SSE
-	else if(basic_synth == synth_1to1_s32_sse) type = sse;
+#if defined(OPT_SSE) || defined(OPT_SSE_VINTAGE)
+	else if(basic_synth == synth_1to1_s32_sse)
+	{
+		type = sse_or_vintage(fr);
+	}
 #endif
 #ifdef OPT_X86_64
 	else if(basic_synth == synth_1to1_s32_x86_64) type = x86_64;
 #endif
+#ifdef OPT_AVX
+	else if(basic_synth == synth_1to1_s32_avx) type = avx;
+#endif
 #ifdef OPT_ALTIVEC
 	else if(basic_synth == synth_1to1_s32_altivec) type = altivec;
 #endif
@@ -407,8 +433,10 @@ int set_synth_functions(mpg123_handle *fr)
 #	endif
 #	ifdef ACCURATE_ROUNDING
 	   && fr->cpu_opts.type != sse
+	   && fr->cpu_opts.type != sse_vintage
 	   && fr->cpu_opts.type != x86_64
 	   && fr->cpu_opts.type != neon
+	   && fr->cpu_opts.type != avx
 #	endif
 	  )
 	{
@@ -464,28 +492,31 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #endif
 
 	fr->cpu_opts.type = nodec;
-	/* covers any i386+ cpu; they actually differ only in the synth_1to1 function, mostly... */
-#ifdef OPT_X86
-
 #ifdef OPT_MULTI
 #ifndef NO_LAYER3
-#if (defined OPT_3DNOW || defined OPT_3DNOWEXT)
+#if (defined OPT_3DNOW_VINTAGE || defined OPT_3DNOWEXT_VINTAGE || defined OPT_SSE || defined OPT_X86_64 || defined OPT_AVX)
 	fr->cpu_opts.the_dct36 = dct36;
 #endif
 #endif
 #endif
-
+	/* covers any i386+ cpu; they actually differ only in the synth_1to1 function, mostly... */
+#ifdef OPT_X86
 	if(cpu_i586(cpu_flags))
 	{
 #		ifdef OPT_MULTI
 		debug2("standard flags: 0x%08x\textended flags: 0x%08x", cpu_flags.std, cpu_flags.ext);
 #		endif
-		#ifdef OPT_SSE
+#		ifdef OPT_SSE
 		if(   !done && (auto_choose || want_dec == sse)
 		   && cpu_sse(cpu_flags) && cpu_mmx(cpu_flags) )
 		{
-			chosen = "SSE";
+			chosen = dn_sse;
 			fr->cpu_opts.type = sse;
+#ifdef OPT_MULTI
+#			ifndef NO_LAYER3
+			/* if(cpu_fast_sse(cpu_flags)) */ fr->cpu_opts.the_dct36 = dct36_sse;
+#			endif
+#endif
 #			ifndef NO_16BIT
 			fr->synths.plain[r_1to1][f_16] = synth_1to1_sse;
 #			ifdef ACCURATE_ROUNDING
@@ -502,15 +533,52 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #			endif
 			done = 1;
 		}
-		#endif
+#		endif
+#		ifdef OPT_SSE_VINTAGE
+		if(   !done && (auto_choose || want_dec == sse_vintage)
+		   && cpu_sse(cpu_flags) && cpu_mmx(cpu_flags) )
+		{
+			chosen = dn_sse_vintage;
+			fr->cpu_opts.type = sse_vintage;
+#			ifndef NO_16BIT
+			fr->synths.plain[r_1to1][f_16] = synth_1to1_sse;
+#			ifdef ACCURATE_ROUNDING
+			fr->synths.stereo[r_1to1][f_16] = synth_1to1_stereo_sse;
+#			endif
+#			endif
+#			ifndef NO_REAL
+			fr->synths.plain[r_1to1][f_real] = synth_1to1_real_sse;
+			fr->synths.stereo[r_1to1][f_real] = synth_1to1_real_stereo_sse;
+#			endif
+#			ifndef NO_32BIT
+			fr->synths.plain[r_1to1][f_32] = synth_1to1_s32_sse;
+			fr->synths.stereo[r_1to1][f_32] = synth_1to1_s32_stereo_sse;
+#			endif
+			done = 1;
+		}
+#		endif
 #		ifdef OPT_3DNOWEXT
-		if(   !done && (auto_choose || want_dec == dreidnowext )
+		if(   !done && (auto_choose || want_dec == dreidnowext)
 		   && cpu_3dnow(cpu_flags)
 		   && cpu_3dnowext(cpu_flags)
 		   && cpu_mmx(cpu_flags) )
 		{
-			chosen = "3DNowExt";
+			chosen = dn_dreidnowext;
 			fr->cpu_opts.type = dreidnowext;
+#			ifndef NO_16BIT
+			fr->synths.plain[r_1to1][f_16] = synth_1to1_3dnowext;
+#			endif
+			done = 1;
+		}
+#		endif
+#		ifdef OPT_3DNOWEXT_VINTAGE
+		if(   !done && (auto_choose || want_dec == dreidnowext_vintage)
+		   && cpu_3dnow(cpu_flags)
+		   && cpu_3dnowext(cpu_flags)
+		   && cpu_mmx(cpu_flags) )
+		{
+			chosen = dn_dreidnowext_vintage;
+			fr->cpu_opts.type = dreidnowext_vintage;
 #ifdef OPT_MULTI
 #			ifndef NO_LAYER3
 			fr->cpu_opts.the_dct36 = dct36_3dnowext;
@@ -521,13 +589,25 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #			endif
 			done = 1;
 		}
-		#endif
-		#ifdef OPT_3DNOW
+#		endif
+#		ifdef OPT_3DNOW
 		if(    !done && (auto_choose || want_dec == dreidnow)
 		    && cpu_3dnow(cpu_flags) && cpu_mmx(cpu_flags) )
 		{
-			chosen = "3DNow";
+			chosen = dn_dreidnow;
 			fr->cpu_opts.type = dreidnow;
+#			ifndef NO_16BIT
+			fr->synths.plain[r_1to1][f_16] = synth_1to1_3dnow;
+#			endif
+			done = 1;
+		}
+#		endif
+#		ifdef OPT_3DNOW_VINTAGE
+		if(    !done && (auto_choose || want_dec == dreidnow_vintage)
+		    && cpu_3dnow(cpu_flags) && cpu_mmx(cpu_flags) )
+		{
+			chosen = dn_dreidnow_vintage;
+			fr->cpu_opts.type = dreidnow_vintage;
 #ifdef OPT_MULTI
 #			ifndef NO_LAYER3
 			fr->cpu_opts.the_dct36 = dct36_3dnow;
@@ -538,12 +618,12 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #			endif
 			done = 1;
 		}
-		#endif
+#		endif
 		#ifdef OPT_MMX
 		if(   !done && (auto_choose || want_dec == mmx)
 		   && cpu_mmx(cpu_flags) )
 		{
-			chosen = "MMX";
+			chosen = dn_mmx;
 			fr->cpu_opts.type = mmx;
 #			ifndef NO_16BIT
 			fr->synths.plain[r_1to1][f_16] = synth_1to1_mmx;
@@ -584,7 +664,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 	   But still... here it is... maybe for real use in future. */
 	if(!done && (auto_choose || want_dec == ivier))
 	{
-		chosen = "i486";
+		chosen = dn_ivier;
 		fr->cpu_opts.type = ivier;
 		done = 1;
 	}
@@ -592,7 +672,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 	#ifdef OPT_I386
 	if(!done && (auto_choose || want_dec == idrei))
 	{
-		chosen = "i386";
+		chosen = dn_idrei;
 		fr->cpu_opts.type = idrei;
 		done = 1;
 	}
@@ -627,11 +707,42 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 
 #endif /* OPT_X86 */
 
+#ifdef OPT_AVX
+	if(!done && (auto_choose || want_dec == avx) && cpu_avx(cpu_flags))
+	{
+		chosen = "x86-64 (AVX)";
+		fr->cpu_opts.type = avx;
+#ifdef OPT_MULTI
+#		ifndef NO_LAYER3
+		fr->cpu_opts.the_dct36 = dct36_avx;
+#		endif
+#endif
+#		ifndef NO_16BIT
+		fr->synths.plain[r_1to1][f_16] = synth_1to1_avx;
+		fr->synths.stereo[r_1to1][f_16] = synth_1to1_stereo_avx;
+#		endif
+#		ifndef NO_REAL
+		fr->synths.plain[r_1to1][f_real] = synth_1to1_real_avx;
+		fr->synths.stereo[r_1to1][f_real] = synth_1to1_real_stereo_avx;
+#		endif
+#		ifndef NO_32BIT
+		fr->synths.plain[r_1to1][f_32] = synth_1to1_s32_avx;
+		fr->synths.stereo[r_1to1][f_32] = synth_1to1_s32_stereo_avx;
+#		endif
+		done = 1;
+	}
+#endif
+
 #ifdef OPT_X86_64
 	if(!done && (auto_choose || want_dec == x86_64))
 	{
 		chosen = "x86-64 (SSE)";
 		fr->cpu_opts.type = x86_64;
+#ifdef OPT_MULTI
+#		ifndef NO_LAYER3
+		fr->cpu_opts.the_dct36 = dct36_x86_64;
+#		endif
+#endif
 #		ifndef NO_16BIT
 		fr->synths.plain[r_1to1][f_16] = synth_1to1_x86_64;
 		fr->synths.stereo[r_1to1][f_16] = synth_1to1_stereo_x86_64;
@@ -668,7 +779,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #	ifdef OPT_ALTIVEC
 	if(!done && (auto_choose || want_dec == altivec))
 	{
-		chosen = "AltiVec";
+		chosen = dn_altivec;
 		fr->cpu_opts.type = altivec;
 #		ifndef NO_16BIT
 		fr->synths.plain[r_1to1][f_16] = synth_1to1_altivec;
@@ -689,7 +800,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #	ifdef OPT_NEON
 	if(!done && (auto_choose || want_dec == neon))
 	{
-		chosen = "NEON";
+		chosen = dn_neon;
 		fr->cpu_opts.type = neon;
 #		ifndef NO_16BIT
 		fr->synths.plain[r_1to1][f_16] = synth_1to1_neon;
@@ -710,7 +821,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #	ifdef OPT_ARM
 	if(!done && (auto_choose || want_dec == arm))
 	{
-		chosen = "ARM";
+		chosen = dn_arm;
 		fr->cpu_opts.type = arm;
 #		ifndef NO_16BIT
 		fr->synths.plain[r_1to1][f_16] = synth_1to1_arm;
@@ -722,7 +833,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #	ifdef OPT_GENERIC
 	if(!done && (auto_choose || want_dec == generic))
 	{
-		chosen = "generic";
+		chosen = dn_generic;
 		fr->cpu_opts.type = generic;
 		done = 1;
 	}
@@ -789,12 +900,21 @@ static const char *mpg123_supported_decoder_list[] =
 	#ifdef OPT_SSE
 	NULL,
 	#endif
+	#ifdef OPT_SSE_VINTAGE
+	NULL,
+	#endif
 	#ifdef OPT_3DNOWEXT
 	NULL,
 	#endif
+	#ifdef OPT_3DNOWEXT_VINTAGE
+	NULL,
+	#endif
 	#ifdef OPT_3DNOW
 	NULL,
 	#endif
+	#ifdef OPT_3DNOW_VINTAGE
+	NULL,
+	#endif
 	#ifdef OPT_MMX
 	NULL,
 	#endif
@@ -813,6 +933,9 @@ static const char *mpg123_supported_decoder_list[] =
 	#ifdef OPT_ALTIVEC
 	NULL,
 	#endif
+	#ifdef OPT_AVX
+	NULL,
+	#endif
 	#ifdef OPT_X86_64
 	NULL,
 	#endif
@@ -838,40 +961,52 @@ static const char *mpg123_supported_decoder_list[] =
 static const char *mpg123_decoder_list[] =
 {
 	#ifdef OPT_SSE
-	dn_SSE,
+	dn_sse,
+	#endif
+	#ifdef OPT_SSE_VINTAGE
+	dn_sse_vintage,
 	#endif
 	#ifdef OPT_3DNOWEXT
-	dn_3DNowExt,
+	dn_dreidnowext,
+	#endif
+	#ifdef OPT_3DNOWEXT_VINTAGE
+	dn_dreidnowext_vintage,
 	#endif
 	#ifdef OPT_3DNOW
-	dn_3DNow,
+	dn_dreidnow,
+	#endif
+	#ifdef OPT_3DNOW_VINTAGE
+	dn_dreidnow_vintage,
 	#endif
 	#ifdef OPT_MMX
-	dn_MMX,
+	dn_mmx,
 	#endif
 	#ifdef OPT_I586
-	dn_i586,
+	dn_ifuenf,
 	#endif
 	#ifdef OPT_I586_DITHER
-	dn_i586_dither,
+	dn_ifuenf_dither,
 	#endif
 	#ifdef OPT_I486
-	dn_i486,
+	dn_ivier,
 	#endif
 	#ifdef OPT_I386
-	dn_i386,
+	dn_idrei,
 	#endif
 	#ifdef OPT_ALTIVEC
-	dn_AltiVec,
+	dn_altivec,
+	#endif
+	#ifdef OPT_AVX
+	dn_avx,
 	#endif
 	#ifdef OPT_X86_64
 	dn_x86_64,
 	#endif
 	#ifdef OPT_ARM
-	dn_ARM,
+	dn_arm,
 	#endif
 	#ifdef OPT_NEON
-	dn_NEON,
+	dn_neon,
 	#endif
 	#ifdef OPT_GENERIC
 	dn_generic,
@@ -889,57 +1024,71 @@ void check_decoders(void )
 	return;
 #else
 	const char **d = mpg123_supported_decoder_list;
-#ifdef OPT_X86
+#if (defined OPT_X86) || (defined OPT_X86_64)
 	getcpuflags(&cpu_flags);
+#endif
+#ifdef OPT_X86
 	if(cpu_i586(cpu_flags))
 	{
 		/* not yet: if(cpu_sse2(cpu_flags)) printf(" SSE2");
 		if(cpu_sse3(cpu_flags)) printf(" SSE3"); */
 #ifdef OPT_SSE
-		if(cpu_sse(cpu_flags)) *(d++) = decname[sse];
+		if(cpu_sse(cpu_flags)) *(d++) = dn_sse;
+#endif
+#ifdef OPT_SSE_VINTAGE
+		if(cpu_sse(cpu_flags)) *(d++) = dn_sse_vintage;
 #endif
 #ifdef OPT_3DNOWEXT
-		if(cpu_3dnowext(cpu_flags)) *(d++) = decname[dreidnowext];
+		if(cpu_3dnowext(cpu_flags)) *(d++) = dn_dreidnowext;
+#endif
+#ifdef OPT_3DNOWEXT_VINTAGE
+		if(cpu_3dnowext(cpu_flags)) *(d++) = dn_dreidnowext_vintage;
 #endif
 #ifdef OPT_3DNOW
-		if(cpu_3dnow(cpu_flags)) *(d++) = decname[dreidnow];
+		if(cpu_3dnow(cpu_flags)) *(d++) = dn_dreidnow;
+#endif
+#ifdef OPT_3DNOW_VINTAGE
+		if(cpu_3dnow(cpu_flags)) *(d++) = dn_dreidnow_vintage;
 #endif
 #ifdef OPT_MMX
-		if(cpu_mmx(cpu_flags)) *(d++) = decname[mmx];
+		if(cpu_mmx(cpu_flags)) *(d++) = dn_mmx;
 #endif
 #ifdef OPT_I586
-		*(d++) = decname[ifuenf];
+		*(d++) = dn_ifuenf;
 #endif
 #ifdef OPT_I586_DITHER
-		*(d++) = decname[ifuenf_dither];
+		*(d++) = dn_ifuenf_dither;
 #endif
 	}
 #endif
 /* just assume that the i486 built is run on a i486 cpu... */
 #ifdef OPT_I486
-	*(d++) = decname[ivier];
+	*(d++) = dn_ivier;
 #endif
 #ifdef OPT_ALTIVEC
-	*(d++) = decname[altivec];
+	*(d++) = dn_altivec;
 #endif
 /* every supported x86 can do i386, any cpu can do generic */
 #ifdef OPT_I386
-	*(d++) = decname[idrei];
+	*(d++) = dn_idrei;
+#endif
+#ifdef OPT_AVX
+	if(cpu_avx(cpu_flags)) *(d++) = dn_avx;
 #endif
 #ifdef OPT_X86_64
-	*(d++) = decname[x86_64];
+	*(d++) = dn_x86_64;
 #endif
 #ifdef OPT_ARM
-	*(d++) = decname[arm];
+	*(d++) = dn_arm;
 #endif
 #ifdef OPT_NEON
-	*(d++) = decname[neon];
+	*(d++) = dn_neon;
 #endif
 #ifdef OPT_GENERIC
-	*(d++) = decname[generic];
+	*(d++) = dn_generic;
 #endif
 #ifdef OPT_GENERIC_DITHER
-	*(d++) = decname[generic_dither];
+	*(d++) = dn_generic_dither;
 #endif
 #endif /* ndef OPT_MULTI */
 }
