@@ -27,80 +27,101 @@ static int term_enable = 0;
 static struct termios old_tio;
 int seeking = FALSE;
 
+/* Buffered key from a signal or whatnot.
+   We ignore the null character... */
+static char prekey = 0;
+
 /* Hm, next step would be some system in this, plus configurability...
    Two keys for everything? It's just stop/pause for now... */
 struct keydef { const char key; const char key2; const char* desc; };
 struct keydef term_help[] =
 {
-	 { STOP_KEY,  ' ', "interrupt/restart playback (i.e. '(un)pause')" }
-	,{ NEXT_KEY,    0, "next track" }
-	,{ PREV_KEY,    0, "previous track" }
-	,{ BACK_KEY,    0, "back to beginning of track" }
-	,{ PAUSE_KEY,   0, "loop around current position (like a damaged audio CD;-)" }
-	,{ FORWARD_KEY, 0, "forward" }
-	,{ REWIND_KEY,  0, "rewind" }
-	,{ FAST_FORWARD_KEY, 0, "fast forward" }
-	,{ FAST_REWIND_KEY,  0, "fast rewind" }
-	,{ FINE_FORWARD_KEY, 0, "fine forward" }
-	,{ FINE_REWIND_KEY,  0, "fine rewind" }
-	,{ VOL_UP_KEY,   0, "volume up" }
-	,{ VOL_DOWN_KEY, 0, "volume down" }
-	,{ RVA_KEY,      0, "RVA switch" }
-	,{ VERBOSE_KEY,  0, "verbose switch" }
-	,{ PLAYLIST_KEY, 0, "list current playlist, indicating current track there" }
-	,{ TAG_KEY,      0, "display tag info (again)" }
-	,{ MPEG_KEY,     0, "print MPEG header info (again)" }
-	,{ HELP_KEY,     0, "this help" }
-	,{ QUIT_KEY,     0, "quit" }
-	,{ PITCH_UP_KEY, PITCH_BUP_KEY, "pitch up (small step, big step)" }
-	,{ PITCH_DOWN_KEY, PITCH_BDOWN_KEY, "pitch down (small step, big step)" }
-	,{ PITCH_ZERO_KEY, 0, "reset pitch to zero" }
+	 { MPG123_STOP_KEY,  ' ', "interrupt/restart playback (i.e. '(un)pause')" }
+	,{ MPG123_NEXT_KEY,    0, "next track" }
+	,{ MPG123_PREV_KEY,    0, "previous track" }
+	,{ MPG123_BACK_KEY,    0, "back to beginning of track" }
+	,{ MPG123_PAUSE_KEY,   0, "loop around current position (don't combine with output buffer)" }
+	,{ MPG123_FORWARD_KEY, 0, "forward" }
+	,{ MPG123_REWIND_KEY,  0, "rewind" }
+	,{ MPG123_FAST_FORWARD_KEY, 0, "fast forward" }
+	,{ MPG123_FAST_REWIND_KEY,  0, "fast rewind" }
+	,{ MPG123_FINE_FORWARD_KEY, 0, "fine forward" }
+	,{ MPG123_FINE_REWIND_KEY,  0, "fine rewind" }
+	,{ MPG123_VOL_UP_KEY,   0, "volume up" }
+	,{ MPG123_VOL_DOWN_KEY, 0, "volume down" }
+	,{ MPG123_RVA_KEY,      0, "RVA switch" }
+	,{ MPG123_VERBOSE_KEY,  0, "verbose switch" }
+	,{ MPG123_PLAYLIST_KEY, 0, "list current playlist, indicating current track there" }
+	,{ MPG123_TAG_KEY,      0, "display tag info (again)" }
+	,{ MPG123_MPEG_KEY,     0, "print MPEG header info (again)" }
+	,{ MPG123_HELP_KEY,     0, "this help" }
+	,{ MPG123_QUIT_KEY,     0, "quit" }
+	,{ MPG123_PITCH_UP_KEY, MPG123_PITCH_BUP_KEY, "pitch up (small step, big step)" }
+	,{ MPG123_PITCH_DOWN_KEY, MPG123_PITCH_BDOWN_KEY, "pitch down (small step, big step)" }
+	,{ MPG123_PITCH_ZERO_KEY, 0, "reset pitch to zero" }
+	,{ MPG123_BOOKMARK_KEY, 0, "print out current position in playlist and track, for the benefit of some external tool to store bookmarks" }
 };
 
 void term_sigcont(int sig);
+static void term_sigusr(int sig);
 
 /* This must call only functions safe inside a signal handler. */
 int term_setup(struct termios *pattern)
 {
-  struct termios tio = *pattern;
+	struct termios tio = *pattern;
 
-  signal(SIGCONT, term_sigcont);
+	/* One might want to use sigaction instead. */
+	signal(SIGCONT, term_sigcont);
+	signal(SIGUSR1, term_sigusr);
+	signal(SIGUSR2, term_sigusr);
 
-  tio.c_lflag &= ~(ICANON|ECHO); 
-  tio.c_cc[VMIN] = 1;
-  tio.c_cc[VTIME] = 0;
-  return tcsetattr(0,TCSANOW,&tio);
+	tio.c_lflag &= ~(ICANON|ECHO); 
+	tio.c_cc[VMIN] = 1;
+	tio.c_cc[VTIME] = 0;
+	return tcsetattr(0,TCSANOW,&tio);
 }
 
 void term_sigcont(int sig)
 {
-  term_enable = 0;
+	term_enable = 0;
 
-  if (term_setup(&old_tio) < 0) {
-    fprintf(stderr,"Can't set terminal attributes\n");
-    return;
-  }
+	if (term_setup(&old_tio) < 0)
+	{
+		fprintf(stderr,"Can't set terminal attributes\n");
+		return;
+	}
 
-  term_enable = 1;
+	term_enable = 1;
+}
+
+static void term_sigusr(int sig)
+{
+	switch(sig)
+	{
+		case SIGUSR1: prekey=*param.term_usr1; break;
+		case SIGUSR2: prekey=*param.term_usr2; break;
+	}
 }
 
 /* initialze terminal */
 void term_init(void)
 {
-  debug("term_init");
+	debug("term_init");
 
-  term_enable = 0;
+	term_enable = 0;
 
-  if(tcgetattr(0,&old_tio) < 0) {
-    fprintf(stderr,"Can't get terminal attributes\n");
-    return;
-  }
-  if(term_setup(&old_tio) < 0) {
-    fprintf(stderr,"Can't set terminal attributes\n");
-    return;
-  }
+	if(tcgetattr(0,&old_tio) < 0)
+	{
+		fprintf(stderr,"Can't get terminal attributes\n");
+		return;
+	}
+	if(term_setup(&old_tio) < 0)
+	{
+		fprintf(stderr,"Can't set terminal attributes\n");
+		return;
+	}
 
-  term_enable = 1;
+	term_enable = 1;
 }
 
 void term_hint(void)
@@ -205,33 +226,47 @@ static void seekmode(void)
 	{
 		stopped = TRUE;
 		buffer_stop();
-		fprintf(stderr, "%s", STOPPED_STRING);
+		fprintf(stderr, "%s", MPG123_STOPPED_STRING);
 	}
 }
 
-static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_delay)
+/* Get the next pressed key, if any.
+   Returns 1 when there is a key, 0 if not. */
+static int get_key(int do_delay, char *val)
 {
-  int n = 1;
-  /* long offset = 0; */
-  
-  while(n > 0) {
-    fd_set r;
-    struct timeval t;
-    char val;
+	fd_set r;
+	struct timeval t;
 
-    t.tv_sec=0;
-    t.tv_usec=(do_delay) ? 10*1000 : 0;
-    
-    FD_ZERO(&r);
-    FD_SET(0,&r);
-    n = select(1,&r,NULL,NULL,&t);
-    if(n > 0 && FD_ISSET(0,&r)) {
-      if(read(0,&val,1) <= 0)
-        break;
+	/* Shortcut: If some other means sent a key, use it. */
+	if(prekey)
+	{
+		debug1("Got prekey: %c\n", prekey);
+		*val = prekey;
+		prekey = 0;
+		return 1;
+	}
 
-      switch(tolower(val)) {
-	case BACK_KEY:
-        if(!param.usebuffer) ao->flush(ao);
+	t.tv_sec=0;
+	t.tv_usec=(do_delay) ? 10*1000 : 0;
+
+	FD_ZERO(&r);
+	FD_SET(0,&r);
+	if(select(1,&r,NULL,NULL,&t) > 0 && FD_ISSET(0,&r))
+	{
+		if(read(0,val,1) <= 0)
+		return 0; /* Well, we couldn't read the key, so there is none. */
+		else
+		return 1;
+	}
+	else return 0;
+}
+
+static void term_handle_key(mpg123_handle *fr, audio_output_t *ao, char val)
+{
+	switch(tolower(val))
+	{
+	case MPG123_BACK_KEY:
+		if(!param.usebuffer) ao->flush(ao);
 				else buffer_resync();
 		if(paused) pause_cycle=(int)(LOOP_CYCLES/mpg123_tpf(fr));
 
@@ -239,13 +274,13 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 		error1("Seek to begin failed: %s", mpg123_strerror(fr));
 
 		framenum=0;
-		break;
-	case NEXT_KEY:
+	break;
+	case MPG123_NEXT_KEY:
 		if(!param.usebuffer) ao->flush(ao);
 		else buffer_resync(); /* was: plain_buffer_resync */
-	  next_track();
-	  break;
-	case QUIT_KEY:
+		next_track();
+	break;
+	case MPG123_QUIT_KEY:
 		debug("QUIT");
 		if(stopped)
 		{
@@ -258,32 +293,32 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 		}
 		set_intflag();
 		offset = 0;
-	  break;
-	case PAUSE_KEY:
-  	  paused=1-paused;
-	  if(paused) {
+	break;
+	case MPG123_PAUSE_KEY:
+		paused=1-paused;
+		if(paused) {
 			/* Not really sure if that is what is wanted
-			   This jumps in audio output, but has direct reaction to pausing loop. */
+				 This jumps in audio output, but has direct reaction to pausing loop. */
 			if(param.usebuffer) buffer_resync();
 
 			pause_recycle(fr);
-	  }
+		}
 		if(stopped)
 		{
 			stopped=0;
 			if(param.usebuffer) buffer_start();
 		}
-	  fprintf(stderr, "%s", (paused) ? PAUSED_STRING : EMPTY_STRING);
-	  break;
-	case STOP_KEY:
+		fprintf(stderr, "%s", (paused) ? MPG123_PAUSED_STRING : MPG123_EMPTY_STRING);
+	break;
+	case MPG123_STOP_KEY:
 	case ' ':
 		/* when seeking while stopped and then resuming, I want to prevent the chirp from the past */
 		if(!param.usebuffer) ao->flush(ao);
-	  stopped=1-stopped;
-	  if(paused) {
-		  paused=0;
-		  offset -= pause_cycle;
-	  }
+		stopped=1-stopped;
+		if(paused) {
+			paused=0;
+			offset -= pause_cycle;
+		}
 		if(param.usebuffer)
 		{
 			if(stopped) buffer_stop();
@@ -295,87 +330,58 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 				buffer_start();
 			}
 		}
-	  fprintf(stderr, "%s", (stopped) ? STOPPED_STRING : EMPTY_STRING);
-	  break;
-	case FINE_REWIND_KEY:
-	  if(param.usebuffer) seekmode();
-	  offset--;
-	  break;
-	case FINE_FORWARD_KEY:
-	  seekmode();
-	  offset++;
-	  break;
-	case REWIND_KEY:
-	  seekmode();
-  	  offset-=10;
-	  break;
-	case FORWARD_KEY:
-	  seekmode();
-	  offset+=10;
-	  break;
-	case FAST_REWIND_KEY:
-	  seekmode();
-	  offset-=50;
-	  break;
-	case FAST_FORWARD_KEY:
-	  seekmode();
-	  offset+=50;
-	  break;
-	case VOL_UP_KEY:
+		fprintf(stderr, "%s", (stopped) ? MPG123_STOPPED_STRING : MPG123_EMPTY_STRING);
+	break;
+	case MPG123_FINE_REWIND_KEY:
+		if(param.usebuffer) seekmode();
+		offset--;
+	break;
+	case MPG123_FINE_FORWARD_KEY:
+		seekmode();
+		offset++;
+	break;
+	case MPG123_REWIND_KEY:
+		seekmode();
+		  offset-=10;
+	break;
+	case MPG123_FORWARD_KEY:
+		seekmode();
+		offset+=10;
+	break;
+	case MPG123_FAST_REWIND_KEY:
+		seekmode();
+		offset-=50;
+	break;
+	case MPG123_FAST_FORWARD_KEY:
+		seekmode();
+		offset+=50;
+	break;
+	case MPG123_VOL_UP_KEY:
 		mpg123_volume_change(fr, 0.02);
 	break;
-	case VOL_DOWN_KEY:
+	case MPG123_VOL_DOWN_KEY:
 		mpg123_volume_change(fr, -0.02);
 	break;
-	case PITCH_UP_KEY:
-	case PITCH_BUP_KEY:
-	case PITCH_DOWN_KEY:
-	case PITCH_BDOWN_KEY:
-	case PITCH_ZERO_KEY:
+	case MPG123_PITCH_UP_KEY:
+	case MPG123_PITCH_BUP_KEY:
+	case MPG123_PITCH_DOWN_KEY:
+	case MPG123_PITCH_BDOWN_KEY:
+	case MPG123_PITCH_ZERO_KEY:
 	{
-		double old_pitch = param.pitch;
-		long rate;
-		int channels, format;
-		int smode = 0;
-		if(param.usebuffer)
-		{
-			error("No runtime pitch change with output buffer, sorry.");
-			break;
-		}
+		double new_pitch = param.pitch;
 		switch(val) /* Not tolower here! */
 		{
-			case PITCH_UP_KEY:    param.pitch += PITCH_VAL;  break;
-			case PITCH_BUP_KEY:   param.pitch += PITCH_BVAL; break;
-			case PITCH_DOWN_KEY:  param.pitch -= PITCH_VAL;  break;
-			case PITCH_BDOWN_KEY: param.pitch -= PITCH_BVAL; break;
-			case PITCH_ZERO_KEY:  param.pitch = 0.0; break;
+			case MPG123_PITCH_UP_KEY:    new_pitch += MPG123_PITCH_VAL;  break;
+			case MPG123_PITCH_BUP_KEY:   new_pitch += MPG123_PITCH_BVAL; break;
+			case MPG123_PITCH_DOWN_KEY:  new_pitch -= MPG123_PITCH_VAL;  break;
+			case MPG123_PITCH_BDOWN_KEY: new_pitch -= MPG123_PITCH_BVAL; break;
+			case MPG123_PITCH_ZERO_KEY:  new_pitch = 0.0; break;
 		}
-		if(param.pitch < -0.99) param.pitch = -0.99;
-		/* Be safe, check support. */
-		mpg123_getformat(fr, &rate, &channels, &format);
-		if(channels == 1) smode = MPG123_MONO;
-		if(channels == 2) smode = MPG123_STEREO;
-
-		output_pause(ao);
-		audio_capabilities(ao, fr);
-		if(!(mpg123_format_support(fr, rate, format) & smode))
-		{
-			/* Note: When using --pitch command line parameter, you can go higher
-			   because a lower decoder sample rate is automagically chosen.
-			   Here, we'd need to switch decoder rate during track... good? */
-			error("Reached a hardware limit there with pitch!");
-			param.pitch = old_pitch;
-			audio_capabilities(ao, fr);
-		}
-		ao->format   = format;
-		ao->channels = channels;
-		ao->rate     = pitch_rate(rate); 
-		reset_output(ao);
-		output_unpause(ao);
+		set_pitch(fr, ao, new_pitch);
 		fprintf(stderr, "New pitch: %f\n", param.pitch);
 	}
 	break;
-	case VERBOSE_KEY:
+	case MPG123_VERBOSE_KEY:
 		param.verbose++;
 		if(param.verbose > VERBOSE_MAX)
 		{
@@ -384,34 +390,34 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 		}
 		mpg123_param(fr, MPG123_VERBOSE, param.verbose, 0);
 	break;
-	case RVA_KEY:
+	case MPG123_RVA_KEY:
 		if(++param.rva > MPG123_RVA_MAX) param.rva = 0;
 		mpg123_param(fr, MPG123_RVA, param.rva, 0);
 		mpg123_volume_change(fr, 0.);
 	break;
-	case PREV_KEY:
+	case MPG123_PREV_KEY:
 		if(!param.usebuffer) ao->flush(ao);
 		else buffer_resync(); /* was: plain_buffer_resync */
 
 		prev_track();
 	break;
-	case PLAYLIST_KEY:
+	case MPG123_PLAYLIST_KEY:
 		fprintf(stderr, "%s\nPlaylist (\">\" indicates current track):\n", param.verbose ? "\n" : "");
 		print_playlist(stderr, 1);
 		fprintf(stderr, "\n");
 	break;
-	case TAG_KEY:
+	case MPG123_TAG_KEY:
 		fprintf(stderr, "%s\n", param.verbose ? "\n" : "");
 		print_id3_tag(fr, param.long_id3, stderr);
 		fprintf(stderr, "\n");
 	break;
-	case MPEG_KEY:
+	case MPG123_MPEG_KEY:
 		if(param.verbose) print_stat(fr,0,0); /* Make sure that we are talking about the correct frame. */
 		fprintf(stderr, "\n");
 		print_header(fr);
 		fprintf(stderr, "\n");
 	break;
-	case HELP_KEY:
+	case MPG123_HELP_KEY:
 	{ /* This is more than the one-liner before, but it's less spaghetti. */
 		int i;
 		fprintf(stderr,"\n\n -= terminal control keys =-\n");
@@ -422,15 +428,16 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 
 			fprintf(stderr, "\t%s\n", term_help[i].desc);
 		}
+		fprintf(stderr, "\nAlso, the number row (starting at 1, ending at 0) gives you jump points into the current track at 10%% intervals.\n");
 		fprintf(stderr, "\n");
 	}
 	break;
-	case FRAME_INDEX_KEY:
-	case VARIOUS_INFO_KEY:
+	case MPG123_FRAME_INDEX_KEY:
+	case MPG123_VARIOUS_INFO_KEY:
 		if(param.verbose) fprintf(stderr, "\n");
 		switch(val) /* because of tolower() ... */
 		{
-			case FRAME_INDEX_KEY:
+			case MPG123_FRAME_INDEX_KEY:
 			print_index(fr);
 			{
 				long accurate;
@@ -440,7 +447,7 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 				error1("Unable to get state: %s", mpg123_strerror(fr));
 			}
 			break;
-			case VARIOUS_INFO_KEY:
+			case MPG123_VARIOUS_INFO_KEY:
 			{
 				const char* curdec = mpg123_current_decoder(fr);
 				if(curdec == NULL) fprintf(stderr, "Cannot get decoder info!\n");
@@ -448,20 +455,51 @@ static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_dela
 			}
 		}
 	break;
+	case '0':
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9':
+	{
+		off_t len;
+		int num;
+		num = val == '0' ? 10 : val - '0';
+		--num; /* from 0 to 9 */
+
+		seekmode();
+		len = mpg123_length(fr);
+		if(len > 0) mpg123_seek(fr, (off_t)( (num/10.)*len ), SEEK_SET);
+
+	}
+	break;
+	case MPG123_BOOKMARK_KEY:
+		continue_msg("BOOKMARK");
+	break;
 	default:
-	  ;
-      }
-    }
-  }
+		;
+	}
+}
+
+static void term_handle_input(mpg123_handle *fr, audio_output_t *ao, int do_delay)
+{
+	char val;
+	/* Do we really want that while loop? This means possibly handling multiple inputs that come very rapidly in one go. */
+	while(get_key(do_delay, &val))
+	{
+		term_handle_key(fr, ao, val);
+	}
 }
 
 void term_restore(void)
 {
-  
-  if(!term_enable)
-    return;
+	if(!term_enable) return;
 
-  tcsetattr(0,TCSAFLUSH,&old_tio);
+	tcsetattr(0,TCSAFLUSH,&old_tio);
 }
 
 #endif
